/**
 * @license
 * Copyright 2019 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
import * as tf from '@tensorflow/tfjs-core';
export declare type Label = number;
export declare type Color = [number, number, number];
export interface Legend {
    [name: string]: Color;
}
export declare type QuantizationBytes = 1 | 2 | 4;
export declare type ModelArchitecture = 'pascal' | 'cityscapes' | 'ade20k';
export declare type DeepLabInput = ImageData | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | tf.Tensor3D;
export interface ModelConfig {
    quantizationBytes?: QuantizationBytes;
    base?: ModelArchitecture;
    modelUrl?: string;
}
export interface PredictionConfig {
    canvas?: HTMLCanvasElement;
    colormap?: Color[];
    labels?: string[];
}
export interface SegmentationData {
    legend: Legend;
    segmentationMap: Uint8ClampedArray;
}
export interface DeepLabOutput {
    legend: Legend;
    height: number;
    width: number;
    segmentationMap: Uint8ClampedArray;
}
